#!/bin/bash
#
# Process controller script to manage spool jobs defined in a spooltab.
# Spooltab file contains tab-delimited lines defining a job per line.
# See also /etc/default/spooltab.
# Usage: call from a cron job.
# Exit codes: 
#   0 - OK
# 101 - missing spooltab
#
# 2007-03-20 - garancsi.tamas@ionlab.net - kovacs.zoltan@ionlab.net

CAT="/bin/cat"				# cat command call
CHMOD="/bin/chmod"			# chmod command call
CHOWN="/bin/chown"			# chown command call
CUT="/usr/bin/cut"			# cut command call
DATE="/bin/date"			# date command call
FIND="/usr/bin/find"			# find command call
GREP="/bin/grep"		  	# grep command call
PS="/bin/ps"				# ps command call
SU="/bin/su"				# su command call
TOUCH="/bin/touch"			# touch command call
WC="/usr/bin/wc"			# wc command call

LOGFILE="/var/log/spoolprocess.log"	# Logfile pathname
LOGOWNER="root:adm"			# Logfile owner and group
LOGRIGHTS="640"				# Logfile access rights
PROCESS="/usr/local/bin/process.sh"	# Processor script call
SPOOLDEF="/etc/default/spooltab"	# Job defs - a job per line

MSG_DELETED="deleted"
MSG_STARTED="started"

# Getting common settings (if any)
HEADERS="/etc/default/spoolprocess"
. $HEADERS

# Checking mandatory spooltab file
[ -f "$SPOOLDEF" ] || exit 101

# Parsing spool definition file excluding comment lines and empty lines
$CAT $SPOOLDEF 2>/dev/null | \
    $GREP -ve '^#' | \
    $GREP -ve '^[[:space:]]*$' | while read line
do
    # Handling a spooler
    SPOOLUSER=`echo -e "$line" | $CUT -f 1`; [ -z "$SPOOLUSER" ] && SPOOLUSER="root"
    PROCMAX=`echo -e "$line" | $CUT -f 2`; [ -z "$PROCMAX" ] && PROCMAX=1
    SPOOLDIR=`echo -e "$line" | $CUT -f 3`
    PROCESSOR=`echo -e "$line" | $CUT -f 4`
    PROCESSED=`echo -e "$line" | $CUT -f 5`

    # Enumerating spoolfiles from a spool (younger than a day)
    $FIND "$SPOOLDIR" -type f -mtime 0 2>/dev/null | while read spoolfile
    do
	# Checking no. of already running processes
	if [ `$PS aux | $GREP -vi " $GREP " | $GREP "/bin/bash $PROCESS " | $WC -l` -lt $PROCMAX ]; then
	    # Exclude spoolfile when it's now processing
    	    if [ -z "`$PS aux | $GREP -vi " $GREP " | $GREP " $PROCESS $spoolfile "`" ]; then
		# Launch a processor in background
		$SU - $SPOOLUSER -c "$PROCESS \"$spoolfile\" \"$PROCESSOR\" \"$PROCESSED\"" &
		# Create logfile if absent
		if [ ! -f "$LOGFILE" ]; then 
		    $TOUCH $LOGFILE
		    $CHOWN $LOGOWNER $LOGFILE
		    $CHMOD $LOGRIGHTS $LOGFILE
		fi
		# Write a log entry about this processor
		echo "\"`$DATE '+%Y-%m-%d %H:%M:%S'`\" \"$MSG_STARTED\" \"$SPOOLUSER\" \"$PROCESS\" \"$spoolfile\" \"$PROCESSOR\" \"$PROCESSED\"" >>$LOGFILE
	    fi
	fi
    done
done
